@@ -11,7 +11,7 @@
 
 raise "sorry, 1.8 was last decade" unless RUBY_VERSION >= '1.9'
 
-gem 'kramdown', '~> 1.12.0'
+gem 'kramdown', '~> 1.14.0'
 require 'kramdown'
 my_span_elements =  %w{list figure xref eref iref cref spanx vspace}
 Kramdown::Parser::Html::Constants::HTML_SPAN_ELEMENTS.concat my_span_elements
@@ -182,7 +182,7 @@ module Kramdown
               end
             end
           end
-          # compensate for XML2RFC idiosyncracy by insisting on a blank line
+          # compensate for XML2RFC idiosyncrasy by insisting on a blank line
           unless el.attr.delete('tight')
             result[0,0] = "\n" unless result[0,1] == "\n"
           end
@@ -383,6 +383,7 @@ module Kramdown
       end
 
       def convert_a(el, indent, opts)
+        gi = el.attr.delete('gi')
         res = inner(el, indent, opts)
         target = el.attr['target']
         if target[0] == "#"     # handle [](#foo) as xref as in RFC 7328
@@ -390,29 +391,41 @@ module Kramdown
           if target.downcase == res.downcase
             res = ''            # get rid of raw anchors leaking through
           end
-          "<xref#{el_html_attributes(el)}>#{res}</xref>"
+          gi ||= "xref"
         else
-          "<eref#{el_html_attributes(el)}>#{res}</eref>"
+          gi ||= "eref"
         end
+        "<#{gi}#{el_html_attributes(el)}>#{res}</#{gi}>"
       end
 
       def convert_xref(el, indent, opts)
+        gi = el.attr.delete('gi')
         target = el.attr['target']
         if target[0] == "&"
           "#{target};"
-        elsif target =~ %r{\A\w+:(?://|.*@)}
-          "<eref#{el_html_attributes(el)}/>"
         else
-          "<xref#{el_html_attributes(el)}/>"
+          if target =~ %r{\A\w+:(?://|.*@)}
+            gi ||= "eref"
+          else
+            gi ||= "xref"
+          end
+          "<#{gi}#{el_html_attributes(el)}/>"
         end
       end
 
-      REFCACHEDIR = ".refcache"
-      def get_and_cache_resource(url, tn = Time.now, tvalid = 7200)
+      REFCACHEDIR = ENV["KRAMDOWN_REFCACHEDIR"] || ".refcache"
+
+      # warn "*** REFCACHEDIR #{REFCACHEDIR}"
+
+      KRAMDOWN_OFFLINE = ENV["KRAMDOWN_OFFLINE"]
+      KRAMDOWN_REFCACHE_REFETCH = ENV["KRAMDOWN_REFCACHE_REFETCH"]
+
+      # this is now slightly dangerous as multiple urls could map to the same cachefile
+      def get_and_cache_resource(url, cachefile, tvalid = 7200, tn = Time.now)
+        fn = "#{REFCACHEDIR}/#{cachefile}"
         Dir.mkdir(REFCACHEDIR) unless Dir.exists?(REFCACHEDIR)
-        fn = "#{REFCACHEDIR}/#{File.basename(url)}"
-        f = File.stat(fn) rescue nil
-        if !f || tn - f.ctime >= tvalid
+        f = File.stat(fn) rescue nil unless KRAMDOWN_REFCACHE_REFETCH
+        if !KRAMDOWN_OFFLINE && (!f || tn - f.ctime >= tvalid)
           if f
             message = "renewing (stale by #{"%.1f" % ((tn-f.ctime)/86400)} days)"
             fetch_timeout = 10 # seconds, give up quickly if just renewing
@@ -430,12 +443,18 @@ module Kramdown
             end
           else
             require 'open-uri'
+            require 'socket'
+            require 'openssl'
             require 'timeout'
             begin
               Timeout::timeout(fetch_timeout) do
-                open(url) do |f|
-                  s = f.read
-                  if f.status[0] != "200"
+                options = {}
+                if ENV["KRAMDOWN_DONT_VERIFY_HTTPS"]
+                  options[:ssl_verify_mode] = OpenSSL::SSL::VERIFY_NONE
+                end             # workaround for OpenSSL on Windows...
+                open(url, options) do |uf|
+                  s = uf.read
+                  if uf.status[0] != "200"
                     warn "*** Status code #{status} while fetching #{url}"
                   else
                     File.write(fn, s)
@@ -447,21 +466,31 @@ module Kramdown
             end
           end
         end
-        File.read(fn) # this blows up if no cache available after fetch attempt
+        begin
+          File.read(fn) # this blows up if no cache available after fetch attempt
+        rescue Errno::ENOENT => e
+          warn "*** #{e} for ${fn}"
+        end
       end
 
+      # [subdirectory name, cache ttl in seconds, does it provide for ?anchor=]
       XML_RESOURCE_ORG_MAP = {
-        "RFC" => "bibxml", "I-D" => "bibxml3", "W3C" => "bibxml4", "3GPP" => "bibxml5",
+        "RFC" => ["bibxml", 86400*7], # these should change rarely
+        "I-D" => "bibxml3",
+        "W3C" => "bibxml4",
+        "3GPP" => "bibxml5",
         "ANSI" => "bibxml2",
         "CCITT" => "bibxml2",
         "FIPS" => "bibxml2",
-        "IANA" => "bibxml2",
-        "IEEE" => "bibxml2",
+        # "IANA" => "bibxml2",   overtaken by bibxml8
+        "IEEE" => "bibxml2",    # now, how about bibxml6?
         "ISO" => "bibxml2",
         "ITU" => "bibxml2",
         "NIST" => "bibxml2",
         "OASIS" => "bibxml2",
         "PKCS" => "bibxml2",
+        "DOI" => ["bibxml7", 86400, true], # 24 h cache at source anyway
+        "IANA" => ["bibxml8", 86400, true], # ditto
       }
 
       # XML_RESOURCE_ORG_HOST = ENV["XML_RESOURCE_ORG_HOST"] || "xml.resource.org"
@@ -469,26 +498,42 @@ module Kramdown
       XML_RESOURCE_ORG_PREFIX = ENV["XML_RESOURCE_ORG_PREFIX"] ||
                                 "https://#{XML_RESOURCE_ORG_HOST}/public/rfc"
 
+      KRAMDOWN_REFCACHETTL = (e = ENV["KRAMDOWN_REFCACHETTL"]) ? e.to_i : 3600
+
       def convert_img(el, indent, opts) # misuse the tag!
         if a = el.attr
           alt = a.delete('alt').strip
           alt = '' if alt == '!' # work around re-wrap uglyness
-          if anchor = a.delete('src')
-            a['target'] = anchor
+          if src = a.delete('src')
+            a['target'] = src
           end
         end
         if alt == ":include:"   # Really bad misuse of tag...
+          anchor = el.attr.delete('anchor') || (
+            # not yet
+            warn "*** missing anchor for '#{src}'"
+            src
+          )
+          anchor.sub!(/\A[0-9]/) { "_#{$&}" } # can't start an ID with a number
+          anchor.gsub!('/', '_')              # should take out all illegals
           to_insert = ""
-          anchor.scan(/(W3C|3GPP|[A-Z-]+)[.]?([A-Za-z0-9.-]+)/) do |t, n|
+          src.scan(/(W3C|3GPP|[A-Z-]+)[.]?([A-Za-z_0-9.\/\+-]+)/) do |t, n|
             fn = "reference.#{t}.#{n}.xml"
-            sub = XML_RESOURCE_ORG_MAP[t]
+            sub, ttl = XML_RESOURCE_ORG_MAP[t]
+            ttl ||= KRAMDOWN_REFCACHETTL  # everything but RFCs might change a lot
             puts "Huh: ${fn}" unless sub
             url = "#{XML_RESOURCE_ORG_PREFIX}/#{sub}/#{fn}"
-            to_insert = get_and_cache_resource(url)
+            to_insert = get_and_cache_resource(url, fn.gsub('/', '_'), ttl)
             to_insert.scrub! rescue nil # only do this for Ruby >= 2.1
+            # this may be a bit controversial: Don't break the build if reference is broken
+            if KRAMDOWN_OFFLINE
+              to_insert ||= "<reference anchor='#{anchor}'> <front> <title>*** BROKEN REFERENCE ***</title> <author> <organization/> </author> <date/> </front> </reference>"
+            else
+              exit 66 unless to_insert # EX_NOINPUT
+            end
           end
-          to_insert.gsub(/<\?xml version=["']1.0["'] encoding=["']UTF-8["']\?>/, '').
-            gsub(/(anchor=["'])([0-9])/) { "#{$1}_#{$2}"} # can't start an ID with a number
+          to_insert.sub(/<\?xml version=["']1.0["'] encoding=["']UTF-8["']\?>/, '')
+            .sub(/\banchor=(?:"[^"]+"|'[^']+')/, "anchor=\"#{anchor}\"")
         else
           "<xref#{el_html_attributes(el)}>#{alt}</xref>"
         end
